////////////////////////////////////////////////////////////////////
//  ay8910.cpp
//
//	MAMEΥ١ˤߤäΤǤ
//
//  Emulation of the AY-3-8910
//  Based on various code snippets by Ville Hallik, Michael Cuddy,
//  Tatsuyuki Satoh, Fabrice Frances, Nicola Salmoria.
////////////////////////////////////////////////////////////////////
#include <limits.h>
#include <string.h>

#include "ay8910.h"
#include "schedule.h"
#include "z80.h"

// ٥ID
#define	EID_PSG		(1)


#define MAX_OUTPUT	0x7fff
#define STEP 		0x8000

#define AY_AFINE	(0)
#define AY_ACOARSE	(1)
#define AY_BFINE	(2)
#define AY_BCOARSE	(3)
#define AY_CFINE	(4)
#define AY_CCOARSE	(5)
#define AY_NOISEPER	(6)
#define AY_ENABLE	(7)
#define AY_AVOL		(8)
#define AY_BVOL		(9)
#define AY_CVOL		(10)
#define AY_EFINE	(11)
#define AY_ECOARSE	(12)
#define AY_ESHAPE	(13)

#define AY_PORTA	(14)
#define AY_PORTB	(15)

// ݡȥȥå
#define PAWAIT		(1)


////////////////////////////////////////////////////////////////
// 󥹥ȥ饯
////////////////////////////////////////////////////////////////
cAY8910::cAY8910( void ){}
PSG6::PSG6( VM6 *vm, const ID& id ) : P6DEVENT(vm,id), Device(id){}


////////////////////////////////////////////////////////////////
// ǥȥ饯
////////////////////////////////////////////////////////////////
cAY8910::~cAY8910( void ){}
PSG6::~PSG6( void ){}


////////////////////////////////////////////////////////////////
// ٥ȥХåؿ
//
// :	id		٥ID
//			clock	å
// :	ʤ
////////////////////////////////////////////////////////////////
void PSG6::EventCallback( int id, int clock )
{
	switch( id ){
	case EID_PSG:
		break;
	}
}


////////////////////////////////////////////////////////////////
// ݡȥؿ
////////////////////////////////////////////////////////////////
BYTE cAY8910::PortAread( void ){ return 0xff; }
BYTE cAY8910::PortBread( void ){ return 0xff; }
void cAY8910::PortAwrite( BYTE ){}
void cAY8910::PortBwrite( BYTE ){}

BYTE PSG6::PortAread( void ){ return cJoy::GetJoyState(); }
BYTE PSG6::PortBread( void ){ return cJoy::GetJoyState(); }


////////////////////////////////////////////////////////////////
// ץ
//   㴳θޤबѾ(¿ʬ)ʤ
////////////////////////////////////////////////////////////////
int cAY8910::GetUpdateSamples( void ){ return 0; }
int PSG6::GetUpdateSamples( void )
{
	int samples = (int)( (double)vm->snd->GetSampleRate() * vm->sche->Scale( this, EID_PSG ) + 0.5 );
	vm->sche->Reset( this, EID_PSG );
	return samples;
}


////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////
BOOL PSG6::Init( void )
{
	PRINTD( PSG_LOG, "[PSG][Init]\n" );
	
	// PSGå
	cAY8910::SetClock( PSG_CLOCK, vm->snd->GetSampleRate() );
	
	// PSG ܥ塼ơ֥
	cAY8910::SetVolume( DEFAULT_PSGVOL );
	
	// ꥻå
	cAY8910::Reset();
	
	// ȥ꡼³
	vm->snd->ConnectStream( (cRing *)this );
	
//	PRINTD3( PSG_LOG, " event Hz : %10.5fHz ( %d / %d )\n", (double)vm->snd->GetSampleRate() / (double)SndDev::cRing::GetSize(), vm->snd->GetSampleRate(), SndDev::cRing::GetSize() );
//	if( !vm->sche->Add( this, EID_PSG, (double)vm->snd->GetSampleRate() / (double)SndDev::cRing::GetSize(), EV_LOOP|EV_HZ ) ) return FALSE;
	// ʤȤ1ä1󤯤餤ϹȤ
	if( !vm->sche->Add( this, EID_PSG, 1000, EV_LOOP|EV_MS ) ) return FALSE;
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
// PSGå
////////////////////////////////////////////////////////////////
void cAY8910::SetClock( int clock, int rate )
{
	PRINTD( PSG_LOG, "[PSG][SetClock]\n" );
	
	// the step clock for the tone and noise generators is the chip clock
	// divided by 8; for the envelope generator of the AY-3-8910, it is half
	// that much (clock/16), but the envelope of the YM2149 goes twice as
	// fast, therefore again clock/8.
	// Here we calculate the number of steps which happen during one sample
	// at the given sample rate. No. of events = sample rate / (clock/8).
	// STEP is a multiplier used to turn the fraction into a fixed point
	// number.
	UpdateStep = (int)( ((double)STEP * rate * 8) / clock );
}


////////////////////////////////////////////////////////////////
// (ܥ塼ơ֥)
////////////////////////////////////////////////////////////////
void cAY8910::SetVolume( int vol )
{
	PRINTD1( PSG_LOG, "[PSG][SetVolume] %d\n", vol );
	
	SndDev::Volume = min( max( vol, 0 ), 100 );
	
	double out = (MAX_OUTPUT * SndDev::Volume ) / 100;
	
	for( int i=31; i>0; i-- ){
		VolTable[i] = (int)out;
		out /= 1.188502227;	// = 10 ^ (1.5/20) = 1.5dB
	}
	VolTable[0] = 0;
}


////////////////////////////////////////////////////////////////
// ꥻå
////////////////////////////////////////////////////////////////
void cAY8910::Reset( void )
{
	PRINTD( PSG_LOG, "[PSG][Reset]\n" );
	
	RegisterLatch = 0;
	RNG = 1;
	OutputA = 0;
	OutputB = 0;
	OutputC = 0;
	OutputN = 0xff;
	for( int i=0; i<AY_PORTA; i++ ) _AYWriteReg( i, 0 );
		// AYWriteReg() uses the timer system; we cannot
		// call it at this time because the timer system
		// has not been initialized.
}


////////////////////////////////////////////////////////////////
// 쥸
////////////////////////////////////////////////////////////////
// 쥸ߥ
void cAY8910::_AYWriteReg( BYTE r, BYTE v )
{
	int old;
	
	Regs[r] = v;
	
	// A note about the period of tones, noise and envelope: for speed reasons,
	// we count down from the period to 0, but careful studies of the chip
	// output prove that it instead counts up from 0 until the counter becomes
	// greater or equal to the period. This is an important difference when the
	// program is rapidly changing the period to modulate the sound.
	// To compensate for the difference, when the period is changed we adjust
	// our internal counter.
	// Also, note that period = 0 is the same as period = 1. This is mentioned
	// in the YM2203 data sheets. However, this does NOT apply to the Envelope
	// period. In that case, period = 0 is half as period = 1.
	switch( r ){
	case AY_AFINE:
	case AY_ACOARSE:
		Regs[AY_ACOARSE] &= 0x0f;
		old = PeriodA;
		PeriodA = ( Regs[AY_AFINE] + 256 * Regs[AY_ACOARSE] ) * UpdateStep;
		if( PeriodA == 0 ) PeriodA = UpdateStep;
		CountA += PeriodA - old;
		if( CountA <= 0 ) CountA = 1;
		break;
	case AY_BFINE:
	case AY_BCOARSE:
		Regs[AY_BCOARSE] &= 0x0f;
		old = PeriodB;
		PeriodB = ( Regs[AY_BFINE] + 256 * Regs[AY_BCOARSE] ) * UpdateStep;
		if( PeriodB == 0 ) PeriodB = UpdateStep;
		CountB += PeriodB - old;
		if( CountB <= 0 ) CountB = 1;
		break;
	case AY_CFINE:
	case AY_CCOARSE:
		Regs[AY_CCOARSE] &= 0x0f;
		old = PeriodC;
		PeriodC = ( Regs[AY_CFINE] + 256 * Regs[AY_CCOARSE] ) * UpdateStep;
		if( PeriodC == 0 ) PeriodC = UpdateStep;
		CountC += PeriodC - old;
		if( CountC <= 0 ) CountC = 1;
		break;
	case AY_NOISEPER:
		Regs[AY_NOISEPER] &= 0x1f;
		old = PeriodN;
		PeriodN = Regs[AY_NOISEPER] * UpdateStep;
		if( PeriodN == 0 ) PeriodN = UpdateStep;
		CountN += PeriodN - old;
		if( CountN <= 0 ) CountN = 1;
		break;
	case AY_AVOL:
		Regs[AY_AVOL] &= 0x1f;
		EnvelopeA = Regs[AY_AVOL] & 0x10;
		VolA = EnvelopeA ? VolE : VolTable[Regs[AY_AVOL] ? Regs[AY_AVOL]*2+1 : 0];
		break;
	case AY_BVOL:
		Regs[AY_BVOL] &= 0x1f;
		EnvelopeB = Regs[AY_BVOL] & 0x10;
		VolB = EnvelopeB ? VolE : VolTable[Regs[AY_BVOL] ? Regs[AY_BVOL]*2+1 : 0];
		break;
	case AY_CVOL:
		Regs[AY_CVOL] &= 0x1f;
		EnvelopeC = Regs[AY_CVOL] & 0x10;
		VolC = EnvelopeC ? VolE : VolTable[Regs[AY_CVOL] ? Regs[AY_CVOL]*2+1 : 0];
		break;
	case AY_EFINE:
	case AY_ECOARSE:
		old = PeriodE;
		PeriodE = ( (Regs[AY_EFINE] + 256 * Regs[AY_ECOARSE]) ) * UpdateStep;
		if( PeriodE == 0 ) PeriodE = UpdateStep / 2;
		CountE += PeriodE - old;
		if( CountE <= 0 ) CountE = 1;
		break;
	case AY_ESHAPE:
//		envelope shapes:
//		.
//		.
//		.
//		.

//		The envelope counter on the AY-3-8910 has 16 steps. On the YM2149 it
//		has twice the steps, happening twice as fast. Since the end result is
//		just a smoother curve, we always use the YM2149 behaviour.

		Regs[AY_ESHAPE] &= 0x0f;
		Attack = (Regs[AY_ESHAPE] & 0x04) ? 0x1f : 0x00;
		if( (Regs[AY_ESHAPE] & 0x08) == 0 ){
			// if Continue = 0, map the shape to the equivalent one which has Continue = 1
			Hold = 1;
			Alternate = Attack;
		}else{
			Hold = Regs[AY_ESHAPE] & 0x01;
			Alternate = Regs[AY_ESHAPE] & 0x02;
		}
		CountE = PeriodE;
		CountEnv = 0x1f;
		Holding = 0;
		VolE = VolTable[CountEnv ^ Attack];
		if( EnvelopeA ) VolA = VolE;
		if( EnvelopeB ) VolB = VolE;
		if( EnvelopeC ) VolC = VolE;
		break;
	case AY_PORTA:
	// ***  žƤ롩 ***
		if( Regs[AY_ENABLE] & 0x40 ) PortAwrite( v );
		break;
		
	case AY_PORTB:
	// ***  žƤ롩 ***
		if( Regs[AY_ENABLE] & 0x80 ) PortBwrite( v );
		break;
	}
}


// 쥸ߥᥤ
void cAY8910::AYWriteReg( BYTE r, BYTE v )
{
	PRINTD2( PSG_LOG, "[PSG][AYWriteReg] -> %02X, %02X\n", r, v );
	
	if( r > 15 ) return;
	if( r < 14 ){
		if( r == AY_ESHAPE || Regs[r] != v ){
			// 쥸ѹ˥ȥ꡼򹹿
			SoundUpdate( 0 );
		}
	}
	_AYWriteReg( r, v );
}


////////////////////////////////////////////////////////////////
// 쥸ɹ
////////////////////////////////////////////////////////////////
BYTE cAY8910::AYReadReg( BYTE r )
{
	PRINTD1( PSG_LOG, "[PSG][AYReadReg] -> %02X\n", r );
	
	if( r > 15 ) return 0;
	
	switch( r ){
	case AY_PORTA:
	// ***  žƤ롩 ***
		if( !(Regs[AY_ENABLE] & 0x40) ) Regs[AY_PORTA] = PortAread();
		break;
		
	case AY_PORTB:
	// ***  žƤ롩 ***
		if( !(Regs[AY_ENABLE] & 0x80) ) Regs[AY_PORTB] = PortBread();
		break;
	}
	return Regs[r];
}


////////////////////////////////////////////////////////////////
// ȥ꡼๹
//
// :	samples	ץ(-1:ĤХåե 0:åʬ)
// :	int		ץ
////////////////////////////////////////////////////////////////
int cAY8910::SoundUpdate( int samples )
{
	PRINTD2( PSG_LOG, "[PSG][Update] Samples: %d(%d)", samples, SndDev::cRing::FreeSize() );
	
	int length = GetUpdateSamples();
	
	if( samples == 0 )     length = min( length,  SndDev::cRing::FreeSize() );
	else if( samples > 0 ) length = min( samples, SndDev::cRing::FreeSize() );
	else                   length = SndDev::cRing::FreeSize();
	
	PRINTD1( PSG_LOG, " -> %d", length );
	
	
	// The 8910 has three outputs, each output is the mix of one of the three
	// tone generators and of the (single) noise generator. The two are mixed
	// BEFORE going into the DAC. The formula to mix each channel is:
	// (ToneOn | ToneDisable) & (NoiseOn | NoiseDisable).
	// Note that this means that if both tone and noise are disabled, the output
	// is 1, not 0, and can be modulated changing the volume.
	
	// If the channels are disabled, set their output to 1, and increase the
	// counter, if necessary, so they will not be inverted during this update.
	// Setting the output to 1 is necessary because a disabled channel is locked
	// into the ON state (see above); and it has no effect if the volume is 0.
	// If the volume is 0, increase the counter, but don't touch the output.
	if( Regs[AY_ENABLE] & 0x01 ){
		if( CountA <= length*STEP ) CountA += length*STEP;
		OutputA = 1;
	}else if( Regs[AY_AVOL] == 0 ){
		// note that I do count += length, NOT count = length + 1. You might think
		// it's the same since the volume is 0, but doing the latter could cause
		// interferencies when the program is rapidly modulating the volume.
		if( CountA <= length*STEP ) CountA += length*STEP;
	}
	if( Regs[AY_ENABLE] & 0x02 ){
		if( CountB <= length*STEP ) CountB += length*STEP;
		OutputB = 1;
	}else if( Regs[AY_BVOL] == 0 ){
		if( CountB <= length*STEP ) CountB += length*STEP;
	}
	if( Regs[AY_ENABLE] & 0x04 ){
		if( CountC <= length*STEP ) CountC += length*STEP;
		OutputC = 1;
	}
	else if( Regs[AY_CVOL] == 0 ){
		if( CountC <= length*STEP ) CountC += length*STEP;
	}
	
	// for the noise channel we must not touch OutputN - it's also not necessary
	// since we use outn.
	if( (Regs[AY_ENABLE] & 0x38) == 0x38 )	// all off
		if( CountN <= length*STEP ) CountN += length*STEP;
	
	int outn = (OutputN | Regs[AY_ENABLE]);
	
	
	// buffering loop
	for( int i=0; i<length; i++ ){
		int vola,volb,volc;
		int left;
		
		// vola, volb and volc keep track of how long each square wave stays
		// in the 1 position during the sample period.
		vola = volb = volc = 0;
		
		left = STEP;
		do{
			int nextevent;
			
			if( CountN < left ) nextevent = CountN;
			else                nextevent = left;
			
			if( outn & 0x08 ){
				if( OutputA ) vola += CountA;
				CountA -= nextevent;
				// PeriodA is the half period of the square wave. Here, in each
				// loop I add PeriodA twice, so that at the end of the loop the
				// square wave is in the same status (0 or 1) it was at the start.
				// vola is also incremented by PeriodA, since the wave has been 1
				// exactly half of the time, regardless of the initial position.
				// If we exit the loop in the middle, OutputA has to be inverted
				// and vola incremented only if the exit status of the square
				// wave is 1.
				while( CountA <= 0 ){
					CountA += PeriodA;
					if( CountA > 0 ){
						OutputA ^= 1;
						if( OutputA ) vola += PeriodA;
						break;
					}
					CountA += PeriodA;
					vola += PeriodA;
				}
				if( OutputA ) vola -= CountA;
			}else{
				CountA -= nextevent;
				while( CountA <= 0 ){
					CountA += PeriodA;
					if( CountA > 0 ){
						OutputA ^= 1;
						break;
					}
					CountA += PeriodA;
				}
			}
			
			if( outn & 0x10 ){
				if( OutputB ) volb += CountB;
				CountB -= nextevent;
				while( CountB <= 0 ){
					CountB += PeriodB;
					if( CountB > 0 ){
						OutputB ^= 1;
						if( OutputB ) volb += PeriodB;
						break;
					}
					CountB += PeriodB;
					volb += PeriodB;
				}
				if( OutputB ) volb -= CountB;
			}else{
				CountB -= nextevent;
				while( CountB <= 0 ){
					CountB += PeriodB;
					if( CountB > 0 ){
						OutputB ^= 1;
						break;
					}
					CountB += PeriodB;
				}
			}
			
			if( outn & 0x20 ){
				if( OutputC ) volc += CountC;
				CountC -= nextevent;
				while( CountC <= 0 ){
					CountC += PeriodC;
					if( CountC > 0 ){
						OutputC ^= 1;
						if( OutputC ) volc += PeriodC;
						break;
					}
					CountC += PeriodC;
					volc += PeriodC;
				}
				if( OutputC ) volc -= CountC;
			}else{
				CountC -= nextevent;
				while( CountC <= 0 ){
					CountC += PeriodC;
					if( CountC > 0 ){
						OutputC ^= 1;
						break;
					}
					CountC += PeriodC;
				}
			}
			
			CountN -= nextevent;
			if( CountN <= 0 ){
				// Is noise output going to change?
				if( (RNG + 1) & 2 ){	// (bit0^bit1)?
					OutputN = ~OutputN;
					outn = ( OutputN | Regs[AY_ENABLE] );
				}
				
				// The Random Number Generator of the 8910 is a 17-bit shift
				// register. The input to the shift register is bit0 XOR bit2
				// (bit0 is the output).
				
				// The following is a fast way to compute bit 17 = bit0^bit2.
				// Instead of doing all the logic operations, we only check
				// bit 0, relying on the fact that after two shifts of the
				// register, what now is bit 2 will become bit 0, and will
				// invert, if necessary, bit 16, which previously was bit 18.
				if( RNG & 1 ) RNG ^= 0x28000;
				RNG >>= 1;
				CountN += PeriodN;
			}
			
			left -= nextevent;
		}while( left > 0 );
		
		// update envelope
		if( Holding == 0 ){
			CountE -= STEP;
			if( CountE <= 0 ){
				do{
					CountEnv--;
					CountE += PeriodE;
				}while( CountE <= 0 );
				
				// check envelope current position
				if( CountEnv < 0){
					if( Hold ){
						if( Alternate ) Attack ^= 0x1f;
						Holding = 1;
						CountEnv = 0;
					}else{
						// if CountEnv has looped an odd number of times (usually 1),
						// invert the output.
						if( Alternate && (CountEnv & 0x20) ) Attack ^= 0x1f;
						
						CountEnv &= 0x1f;
					}
				}
				
				VolE = VolTable[CountEnv ^ Attack];
				// reload volume
				if( EnvelopeA ) VolA = VolE;
				if( EnvelopeB ) VolB = VolE;
				if( EnvelopeC ) VolC = VolE;
			}
		}
		// Хåե˽
		int odata = (vola * VolA) / STEP + (volb * VolB) / STEP + (volc * VolC) / STEP;
		SndDev::cRing::Put( odata );
	}
	return length;
}


////////////////////////////////////////////////////////////////
// I/Oؿ
////////////////////////////////////////////////////////////////
// PSG쥸ɥ쥹å
void PSG6::OutA0H( int, BYTE data )
{
	vm->cpu->AddWait( PAWAIT );
	cAY8910::RegisterLatch = data & 0x0f;
}

// PSG饤ȥǡ
void PSG6::OutA1H( int, BYTE data )
{
	vm->cpu->AddWait( PAWAIT );
	cAY8910::AYWriteReg( cAY8910::RegisterLatch, data );
}

// PSG󥢥ƥ
void PSG6::OutA3H( int, BYTE data )
{
	vm->cpu->AddWait( PAWAIT );
}

// PSG꡼ɥǡ
BYTE PSG6::InA2H( int )
{
	vm->cpu->AddWait( PAWAIT );
	return cAY8910::AYReadReg( cAY8910::RegisterLatch );
}


////////////////////////////////////////////////////////////////
// ɤǤSAVE
//
// :	Ini		INI֥ȥݥ
// :	BOOL	TRUE: FALSE:
////////////////////////////////////////////////////////////////
BOOL PSG6::DokoSave( cIni *Ini )
{
	cSche::evinfo e;
	char stren[16];
	
	e.device = this;
	
	if( !Ini ) return FALSE;
	
	// cAY8910
	Ini->PutEntry( "PSG", NULL, "RegisterLatch",	"%d",	RegisterLatch );
	for( int i=0; i<16; i++ ){
		char stren[16];
		sprintf( stren, "Regs_%02d", i );
		Ini->PutEntry( "PSG", NULL, stren, "0x%02X", Regs[i] );
	}
	Ini->PutEntry( "PSG", NULL, "PeriodA",		"%d",		PeriodA );
	Ini->PutEntry( "PSG", NULL, "PeriodB",		"%d",		PeriodB );
	Ini->PutEntry( "PSG", NULL, "PeriodC",		"%d",		PeriodC );
	Ini->PutEntry( "PSG", NULL, "PeriodN",		"%d",		PeriodN );
	Ini->PutEntry( "PSG", NULL, "PeriodE",		"%d",		PeriodE );
	Ini->PutEntry( "PSG", NULL, "CountA",		"%d",		CountA );
	Ini->PutEntry( "PSG", NULL, "CountB",		"%d",		CountB );
	Ini->PutEntry( "PSG", NULL, "CountC",		"%d",		CountC );
	Ini->PutEntry( "PSG", NULL, "CountN",		"%d",		CountN );
	Ini->PutEntry( "PSG", NULL, "CountE",		"%d",		CountE );
	Ini->PutEntry( "PSG", NULL, "VolA",			"%d",		VolA );
	Ini->PutEntry( "PSG", NULL, "VolB",			"%d",		VolB );
	Ini->PutEntry( "PSG", NULL, "VolC",			"%d",		VolC );
	Ini->PutEntry( "PSG", NULL, "VolE",			"%d",		VolE );
	Ini->PutEntry( "PSG", NULL, "EnvelopeA",	"0x%02X",	EnvelopeA );
	Ini->PutEntry( "PSG", NULL, "EnvelopeB",	"0x%02X",	EnvelopeB );
	Ini->PutEntry( "PSG", NULL, "EnvelopeC",	"0x%02X",	EnvelopeC );
	Ini->PutEntry( "PSG", NULL, "OutputA",		"0x%02X",	OutputA );
	Ini->PutEntry( "PSG", NULL, "OutputB",		"0x%02X",	OutputB );
	Ini->PutEntry( "PSG", NULL, "OutputC",		"0x%02X",	OutputC );
	Ini->PutEntry( "PSG", NULL, "OutputN",		"0x%02X",	OutputN );
	Ini->PutEntry( "PSG", NULL, "CountEnv",		"%d",		CountEnv );
	Ini->PutEntry( "PSG", NULL, "Hold",			"0x%02X",	Hold );
	Ini->PutEntry( "PSG", NULL, "Alternate",	"0x%02X",	Alternate );
	Ini->PutEntry( "PSG", NULL, "Attack",		"0x%02X",	Attack );
	Ini->PutEntry( "PSG", NULL, "Holding",		"0x%02X",	Holding );
	Ini->PutEntry( "PSG", NULL, "RNG",			"%d",		RNG );
	
	// ٥
	e.id = EID_PSG;
	if( vm->sche->GetEvinfo( &e ) ){
		sprintf( stren, "Event%08X", e.id );
		Ini->PutEntry( "PSG", NULL, stren, "%d %d %d %lf", e.Active ? 1 : 0, e.Period, e.Clock, e.nps );
	}
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
// ɤǤLOAD
//
// :	Ini		INI֥ȥݥ
// :	BOOL	TRUE: FALSE:
////////////////////////////////////////////////////////////////
BOOL PSG6::DokoLoad( cIni *Ini )
{
	int st,yn;
	cSche::evinfo e;
	char stren[16];
	char strrs[64];
	
	e.device = this;
	
	if( !Ini ) return FALSE;
	
	// cAY8910
	Ini->GetInt( "PSG", "RegisterLatch", &RegisterLatch, RegisterLatch );
	for( int i=0; i<16; i++ ){
		char stren[16];
		sprintf( stren, "Regs_%02d", i );
		Ini->GetInt( "PSG", stren, &st, Regs[i] );	Regs[i] = st;
	}
	Ini->GetInt( "PSG", "PeriodA",		&PeriodA,		PeriodA );
	Ini->GetInt( "PSG", "PeriodB",		&PeriodB,		PeriodB );
	Ini->GetInt( "PSG", "PeriodC",		&PeriodC,		PeriodC );
	Ini->GetInt( "PSG", "PeriodN",		&PeriodN,		PeriodN );
	Ini->GetInt( "PSG", "PeriodE",		&PeriodE,		PeriodE );
	Ini->GetInt( "PSG", "CountA",		&CountA,		CountA );
	Ini->GetInt( "PSG", "CountB",		&CountB,		CountB );
	Ini->GetInt( "PSG", "CountC",		&CountC,		CountC );
	Ini->GetInt( "PSG", "CountN",		&CountN,		CountN );
	Ini->GetInt( "PSG", "CountE",		&CountE,		CountE );
	Ini->GetInt( "PSG", "VolA",			&VolA,			VolA );
	Ini->GetInt( "PSG", "VolB",			&VolB,			VolB );
	Ini->GetInt( "PSG", "VolC",			&VolC,			VolC );
	Ini->GetInt( "PSG", "VolE",			&VolE,			VolE );
	Ini->GetInt( "PSG", "EnvelopeA",	&st,			EnvelopeA );	EnvelopeA = st;
	Ini->GetInt( "PSG", "EnvelopeB",	&st,			EnvelopeB );	EnvelopeB = st;
	Ini->GetInt( "PSG", "EnvelopeC",	&st,			EnvelopeC );	EnvelopeC = st;
	Ini->GetInt( "PSG", "OutputA",		&st,			OutputA );		OutputA = st;
	Ini->GetInt( "PSG", "OutputB",		&st,			OutputB );		OutputB = st;
	Ini->GetInt( "PSG", "OutputC",		&st,			OutputC );		OutputC = st;
	Ini->GetInt( "PSG", "OutputN",		&st,			OutputN );		OutputN = st;
	Ini->GetInt( "PSG", "CountEnv",		&st,			CountEnv );		CountEnv = st;
	Ini->GetInt( "PSG", "Hold",			&st,			Hold );			Hold = st;
	Ini->GetInt( "PSG", "Alternate",	&st,			Alternate );	Alternate = st;
	Ini->GetInt( "PSG", "Attack",		&st,			Attack );		Attack = st;
	Ini->GetInt( "PSG", "Holding",		&st,			Holding );		Holding = st;
	Ini->GetInt( "PSG", "RNG",			&RNG,			RNG );
	
	// ٥
	e.id = EID_PSG;
	sprintf( stren, "Event%08X", e.id );
	if( Ini->GetString( "PSG", stren, strrs, "" ) ){
		sscanf( strrs,"%d %d %d %lf", &yn, &e.Period, &e.Clock, &e.nps );
		e.Active = yn ? TRUE : FALSE;
		if( !vm->sche->SetEvinfo( &e ) ) return FALSE;
	}
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
//  device description
////////////////////////////////////////////////////////////////
const Device::Descriptor PSG6::descriptor = {
	PSG6::indef, PSG6::outdef
};

const Device::OutFuncPtr PSG6::outdef[] = {
	STATIC_CAST( Device::OutFuncPtr, &PSG6::OutA0H ),
	STATIC_CAST( Device::OutFuncPtr, &PSG6::OutA1H ),
	STATIC_CAST( Device::OutFuncPtr, &PSG6::OutA3H )
};

const Device::InFuncPtr PSG6::indef[] = {
	STATIC_CAST( Device::InFuncPtr, &PSG6::InA2H )
};
