// ---------------------------------------------------------------------------
//	Virtual Bus Implementation
//	Copyright (c) cisc 1999.
//  ١ˤߤٹ(ά)ΤǤ
// ---------------------------------------------------------------------------
//	$Id: device.cpp,v 1.19 1999/11/26 10:13:24 cisc Exp $

#include <string.h>
#include <new>

#include "pc6001v.h"
#include "device.h"
#include "error.h"


// ---------------------------------------------------------------------------
//	IO Bus
//
IOBus::DummyIO IOBus::dummyio;

IOBus::IOBus()
: ins(0), outs(0), flags(0)
{
}

IOBus::~IOBus()
{
	delete [] ins;
	delete [] outs;
	delete [] flags;
}

////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////
BOOL IOBus::Init( DeviceList* dl, int bs )
{
	devlist  = dl;
	banksize = bs;
	
	// ˻ǴꤤޤƤ
	if( ins ){   delete [] ins;   ins   = NULL; }
	if( outs ){  delete [] outs;  outs  = NULL; }
	if( flags ){ delete [] flags; flags = NULL; }
	
	// 
	try{
		ins   = new InBank[banksize];
		outs  = new OutBank[banksize];
		flags = new BYTE[banksize];
	}
	catch( std::bad_alloc ){	// new ˼Ԥ
		Error::SetError( Error::MemAllocFailed );
		if( ins ){   delete [] ins;   ins   = NULL; }
		if( outs ){  delete [] outs;  outs  = NULL; }
		return FALSE;
	}
	
	ZeroMemory( flags, banksize );
	
	// ȤꤢƤΥݡȤ˥ߡǥХƤ
	for( int i=0; i<banksize; i++ ){
		ins[i].device  = &dummyio;
		ins[i].func    = STATIC_CAST( InFuncPtr, &DummyIO::dummyin );
		ins[i].next    = 0;
		outs[i].device = &dummyio;
		outs[i].func   = STATIC_CAST( OutFuncPtr, &DummyIO::dummyout );
		outs[i].next   = 0;
	}
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
// ǥХ³
////////////////////////////////////////////////////////////////
// IN/OUT -----------
BOOL IOBus::Connect( IDevice* device, const Connector* connector )
{
	if( devlist ) devlist->Add(device);
	
	const IDevice::Descriptor* desc = device->GetDesc();
	
	for( ; connector->rule; connector++ ){
		switch( connector->rule & 3 ){
		case portin:
			if( !ConnectIn(connector->bank, device, desc->indef[connector->id]) )
				return FALSE;
			break;
			
		case portout:
			if( !ConnectOut(connector->bank, device, desc->outdef[connector->id]) )
				return FALSE;
			break;
		}
	}
	return TRUE;
}


// IN -----------
BOOL IOBus::ConnectIn( int bank, IDevice* device, InFuncPtr func )
{
	InBank* i = &ins[bank];
	if( i->func == STATIC_CAST( InFuncPtr, &DummyIO::dummyin ) ){
		// ǽ³
		i->device = device;
		i->func   = func;
	}else{
		// 2ܰʹߤ³
		InBank *j;
		try{
			j = new InBank;
		}
		catch( std::bad_alloc ){	// new ˼Ԥ
			Error::SetError( Error::MemAllocFailed );
			return FALSE;
		}
		j->device = device;
		j->func   = func;
		j->next   = i->next;
		i->next   = j;
	}
	return TRUE;
}


// OUT -----------
BOOL IOBus::ConnectOut( int bank, IDevice* device, OutFuncPtr func )
{
	OutBank* i = &outs[bank];
	if( i->func == STATIC_CAST( OutFuncPtr, &DummyIO::dummyout ) ){
		// ǽ³
		i->device = device;
		i->func   = func;
	}else{
		// 2ܰʹߤ³
		OutBank *j;
		try{
			j = new OutBank;
		}
		catch( std::bad_alloc ){	// new ˼Ԥ
			Error::SetError( Error::MemAllocFailed );
			return FALSE;
		}
		j->device = device;
		j->func   = func;
		j->next   = i->next;
		i->next   = j;
	}
	return TRUE;
}


////////////////////////////////////////////////////////////////
// ǥХ
////////////////////////////////////////////////////////////////
BOOL IOBus::Disconnect( IDevice* device )
{
	if( devlist ) devlist->Del(device);
	
	// IN
  	for( int i=0; i<banksize; i++ ){
		InBank* current = &ins[i];
		InBank* referer = 0;
		while( current ){
			InBank* next = current->next;
			if( current->device == device ){
				if( referer ){
					referer->next = next;
					delete current;
				}else{
					// ٤ƥबǽˤä
					if( next ){
						// ΥƥƤʣ̤ƺ
						*current = *next;
						referer = 0;
						delete next;
						continue;
					}else{
						// ΥƥबͣΥƥä
						current->func = STATIC_CAST( InFuncPtr, &DummyIO::dummyin );
					}
				}
			}
			current = next;
		}
	}
	
	// OUT
	for( int i=0; i<banksize; i++ ){
		OutBank* current = &outs[i];
		OutBank* referer = 0;
		while( current ){
			OutBank* next = current->next;
			if( current->device == device ){
				if( referer ){
					referer->next = next;
					delete current;
				}else{
					// ٤ƥबǽˤä
					if( next ){
						// ΥƥƤʣ̤ƺ
						*current = *next;
						referer = 0;
						delete next;
						continue;
					}else{
						// ΥƥबͣΥƥä
						current->func = STATIC_CAST( OutFuncPtr, &DummyIO::dummyout );
					}
				}
			}
			current = next;
		}
	}
	return TRUE;
}


////////////////////////////////////////////////////////////////
// INؿ
////////////////////////////////////////////////////////////////
BYTE IOBus::In( int port )
{
	InBank* list = &ins[port];
	
	BYTE data = 0xff;
	do{
		data &= (list->device->*list->func)( port );
		list = list->next;
	} while ( list );
	return data;
}


////////////////////////////////////////////////////////////////
// OUTؿ
////////////////////////////////////////////////////////////////
void IOBus::Out( int port, BYTE data )
{
	OutBank* list = &outs[port];
	do{
		(list->device->*list->func)( port, data );
		list = list->next;
	} while ( list );
}


////////////////////////////////////////////////////////////////
// ߡǥХ(IN)
////////////////////////////////////////////////////////////////
BYTE IOBus::DummyIO::dummyin( int )
{
	return 0xff;
}


////////////////////////////////////////////////////////////////
// ߡǥХ(OUT)
////////////////////////////////////////////////////////////////
void IOBus::DummyIO::dummyout( int, BYTE )
{
	return;
}




////////////////////////////////////////////////////////////////
// DeviceList
////////////////////////////////////////////////////////////////
DeviceList::~DeviceList()
{
	Cleanup();
}


void DeviceList::Cleanup()
{
	Node* n = node;
	while( n ){
		Node* nx = n->next;
		delete n;
		n = nx;
	}
	node = 0;
}


BOOL DeviceList::Add( IDevice* t )
{
	ID id = t->GetID();
	if( !id ) return FALSE;
	
	Node* n = FindNode(id);
	if( n ){
		n->count++;
		return TRUE;
	}else{
		try{
			n = new Node;
		}
		catch( std::bad_alloc ){	// new ˼Ԥ
			Error::SetError( Error::MemAllocFailed );
			return FALSE;
		}
		n->entry = t, n->next = node, n->count = 1;
		node = n;
		return TRUE;
	}
}


BOOL DeviceList::Del( const ID id )
{
	for( Node** r = &node; *r; r=&((*r)->next) ){
		if( (*r)->entry->GetID() == id ){
			Node* d = *r;
			if( !--d->count ){
				*r = d->next;
				delete d;
			}
			return TRUE;
		}
	}
	return FALSE;
}


IDevice* DeviceList::Find( const ID id )
{
	Node* n = FindNode( id );
	return n ? n->entry : 0;
}


DeviceList::Node* DeviceList::FindNode( const ID id )
{
	for( Node* n = node; n; n=n->next ){
		if( n->entry->GetID() == id )
			return n;
	}
	return 0;
}

