#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <new>

#include "global.h"
#include "common.h"
#include "error.h"
#include "osd.h"


static const P6KeyName P6KeyNameDef[] = {	// P6 ̾
	{ KP6_UNKNOWN,		"K6_UNKNOWN" },
	
	{ KP6_1,			"K6_1" },
	{ KP6_2,			"K6_2" },
	{ KP6_3,			"K6_3" },
	{ KP6_4,			"K6_4" },
	{ KP6_5,			"K6_5" },
	{ KP6_6,			"K6_6" },
	{ KP6_7,			"K6_7" },
	{ KP6_8,			"K6_8" },
	{ KP6_9,			"K6_9" },
	{ KP6_0,			"K6_0" },
	
	{ KP6_A,			"K6_A" },
	{ KP6_B,			"K6_B" },
	{ KP6_C,			"K6_C" },
	{ KP6_D,			"K6_D" },
	{ KP6_E,			"K6_E" },
	{ KP6_F,			"K6_F" },
	{ KP6_G,			"K6_G" },
	{ KP6_H,			"K6_H" },
	{ KP6_I,			"K6_I" },
	{ KP6_J,			"K6_J" },
	{ KP6_K,			"K6_K" },
	{ KP6_L,			"K6_L" },
	{ KP6_M,			"K6_M" },
	{ KP6_N,			"K6_N" },
	{ KP6_O,			"K6_O" },
	{ KP6_P,			"K6_P" },
	{ KP6_Q,			"K6_Q" },
	{ KP6_R,			"K6_R" },
	{ KP6_S,			"K6_S" },
	{ KP6_T,			"K6_T" },
	{ KP6_U,			"K6_U" },
	{ KP6_V,			"K6_V" },
	{ KP6_W,			"K6_W" },
	{ KP6_X,			"K6_X" },
	{ KP6_Y,			"K6_Y" },
	{ KP6_Z,			"K6_Z" },
	
	{ KP6_F1,			"K6_F1" },
	{ KP6_F2,			"K6_F2" },
	{ KP6_F3,			"K6_F3" },
	{ KP6_F4,			"K6_F4" },
	{ KP6_F5,			"K6_F5" },
	
	{ KP6_MINUS,		"K6_MINUS" },
	{ KP6_CARET,		"K6_CARET" },
	{ KP6_YEN,			"K6_YEN" },
	{ KP6_AT,			"K6_AT" },
	{ KP6_LBRACKET,		"K6_LBRACKET" },
	{ KP6_RBRACKET,		"K6_RBRACKET" },
	{ KP6_SEMICOLON,	"K6_SEMICOLON" },
	{ KP6_COLON,		"K6_COLON" },
	{ KP6_COMMA,		"K6_COMMA" },
	{ KP6_PERIOD,		"K6_PERIOD" },
	{ KP6_SLASH,		"K6_SLASH" },
	{ KP6_UNDERSCORE,	"K6_UNDERSCORE" },
	{ KP6_SPACE,		"K6_SPACE" },
	
	{ KP6_ESC,			"K6_ESC" },
	{ KP6_TAB,			"K6_TAB" },
	{ KP6_CTRL,			"K6_CTRL" },
	{ KP6_SHIFT,		"K6_SHIFT" },
	{ KP6_GRAPH,		"K6_GRAPH" },
	{ KP6_HOME,			"K6_HOME" },
	{ KP6_STOP,			"K6_STOP" },
	{ KP6_PAGE,			"K6_PAGE" },
	{ KP6_RETURN,		"K6_RETURN" },
	{ KP6_KANA,			"K6_KANA" },
	{ KP6_INS,			"K6_INS" },
	{ KP6_DEL,			"K6_DEL" },
	
	{ KP6_UP,			"K6_UP" },
	{ KP6_DOWN,			"K6_DOWN" },
	{ KP6_LEFT,			"K6_LEFT" },
	{ KP6_RIGHT,		"K6_RIGHT" },
	
	{ KP6_MODE,			"K6_MODE" },
	{ KP6_CAPS,			"K6_CAPS" },
	
	
	// ƥ󥭡ĥ
	{ KP6_P0,			"K6_P0" },
	{ KP6_P1,			"K6_P1" },
	{ KP6_P2,			"K6_P2" },
	{ KP6_P3,			"K6_P3" },
	{ KP6_P4,			"K6_P4" },
	{ KP6_P5,			"K6_P5" },
	{ KP6_P6,			"K6_P6" },
	{ KP6_P7,			"K6_P7" },
	{ KP6_P8,			"K6_P8" },
	{ KP6_P9,			"K6_P9" },
	{ KP6_PPLUS,		"K6_PPLUS" },
	{ KP6_PMINUS,		"K6_PMINUS" },
	{ KP6_PMULTIPLY,	"K6_PMULTIPLY" },
	{ KP6_PDIVIDE,		"K6_PDIVIDE" },
	{ KP6_PPERIOD,		"K6_PPERIOD" },
	{ KP6_PRETURN,		"K6_PRETURN" },
	
	
	// Ƽﵡǽ
	{ KFN_1,			"K6_FN1" },
	{ KFN_2,			"K6_FN2" },
	{ KFN_3,			"K6_FN3" },
	{ KFN_4,			"K6_FN4" },
	{ KFN_5,			"K6_FN5" },
	{ KFN_6,			"K6_FN6" },
	{ KFN_7,			"K6_FN7" },
	{ KFN_8,			"K6_FN8" },
	{ KFN_9,			"K6_FN9" }
};


static const PCKeyName PCKeyNameDef[] = {	// ۥ ̾
	{ KVC_UNKNOWN,		"K_UNKNOWN" },
	
	{ KVC_1,			"K_1" },
	{ KVC_2,			"K_2" },
	{ KVC_3,			"K_3" },
	{ KVC_4,			"K_4" },
	{ KVC_5,			"K_5" },
	{ KVC_6,			"K_6" },
	{ KVC_7,			"K_7" },
	{ KVC_8,			"K_8" },
	{ KVC_9,			"K_9" },
	{ KVC_0,			"K_0" },
	
	{ KVC_A,			"K_A" },
	{ KVC_B,			"K_B" },
	{ KVC_C,			"K_C" },
	{ KVC_D,			"K_D" },
	{ KVC_E,			"K_E" },
	{ KVC_F,			"K_F" },
	{ KVC_G,			"K_G" },
	{ KVC_H,			"K_H" },
	{ KVC_I,			"K_I" },
	{ KVC_J,			"K_J" },
	{ KVC_K,			"K_K" },
	{ KVC_L,			"K_L" },
	{ KVC_M,			"K_M" },
	{ KVC_N,			"K_N" },
	{ KVC_O,			"K_O" },
	{ KVC_P,			"K_P" },
	{ KVC_Q,			"K_Q" },
	{ KVC_R,			"K_R" },
	{ KVC_S,			"K_S" },
	{ KVC_T,			"K_T" },
	{ KVC_U,			"K_U" },
	{ KVC_V,			"K_V" },
	{ KVC_W,			"K_W" },
	{ KVC_X,			"K_X" },
	{ KVC_Y,			"K_Y" },
	{ KVC_Z,			"K_Z" },
	
	{ KVC_F1,			"K_F1" },
	{ KVC_F2,			"K_F2" },
	{ KVC_F3,			"K_F3" },
	{ KVC_F4,			"K_F4" },
	{ KVC_F5,			"K_F5" },
	{ KVC_F6,			"K_F6" },
	{ KVC_F7,			"K_F7" },
	{ KVC_F8,			"K_F8" },
	{ KVC_F9,			"K_F9" },
	{ KVC_F10,			"K_F10" },
	{ KVC_F11,			"K_F11" },
	{ KVC_F12,			"K_F12" },
	
	{ KVC_MINUS,		"K_MINUS" },
	{ KVC_CARET,		"K_CARET" },
	{ KVC_BACKSPACE,	"K_BACKSPACE" },
	{ KVC_AT,			"K_AT" },
	{ KVC_LBRACKET,		"K_LBRACKET" },
	{ KVC_SEMICOLON,	"K_SEMICOLON" },
	{ KVC_COLON,		"K_COLON" },
	{ KVC_COMMA,		"K_COMMA" },
	{ KVC_PERIOD,		"K_PERIOD" },
	{ KVC_SLASH,		"K_SLASH" },
	{ KVC_SPACE,		"K_SPACE" },
	
	{ KVC_ESC,			"K_ESC" },
	{ KVC_HANZEN,		"K_HANZEN" },
	{ KVC_TAB,			"K_TAB" },
	{ KVC_CAPSLOCK,		"K_CAPSLOCK" },
	{ KVC_ENTER,		"K_ENTER" },
	{ KVC_LCTRL,		"K_LCTRL" },
	{ KVC_RCTRL,		"K_RCTRL" },
	{ KVC_LSHIFT,		"K_LSHIFT" },
	{ KVC_RSHIFT,		"K_RSHIFT" },
	{ KVC_LALT,			"K_LALT" },
	{ KVC_RALT,			"K_RALT" },
	{ KVC_PRINT,		"K_PRINT" },
	{ KVC_SCROLLLOCK,	"K_SCROLLLOCK" },
	{ KVC_PAUSE,		"K_PAUSE" },
	{ KVC_INSERT,		"K_INSERT" },
	{ KVC_DELETE,		"K_DELETE" },
	{ KVC_HOME,			"K_HOME" },
	{ KVC_END,			"K_END" },
	{ KVC_PAGEUP,		"K_PAGEUP" },
	{ KVC_PAGEDOWN,		"K_PAGEDOWN" },
	
	{ KVC_UP,			"K_UP" },
	{ KVC_DOWN,			"K_DOWN" },
	{ KVC_LEFT,			"K_LEFT" },
	{ KVC_RIGHT,		"K_RIGHT" },
	
	{ KVC_P0,			"K_P_0" },
	{ KVC_P1,			"K_P_1" },
	{ KVC_P2,			"K_P_2" },
	{ KVC_P3,			"K_P_3" },
	{ KVC_P4,			"K_P_4" },
	{ KVC_P5,			"K_P_5" },
	{ KVC_P6,			"K_P_6" },
	{ KVC_P7,			"K_P_7" },
	{ KVC_P8,			"K_P_8" },
	{ KVC_P9,			"K_P_9" },
	{ KVC_NUMLOCK,		"K_NUMLOCK" },
	{ KVC_P_PLUS,		"K_P_PLUS" },
	{ KVC_P_MINUS,		"K_P_MINUS" },
	{ KVC_P_MULTIPLY,	"K_P_MULTIPLY" },
	{ KVC_P_DIVIDE,		"K_P_DIVIDE" },
	{ KVC_P_PERIOD,		"K_P_PERIOD" },
	{ KVC_P_ENTER,		"K_P_ENTER" },
	
	// ܸ쥭ܡɤΤ
	{ KVC_YEN,			"K_YEN" },
	{ KVC_RBRACKET,		"K_RBRACKET" },
	{ KVC_UNDERSCORE,	"K_UNDERSCORE" },
	{ KVC_MUHENKAN,		"K_MUHENKAN" },
	{ KVC_HENKAN,		"K_HENKAN" },
	{ KVC_HIRAGANA,		"K_HIRAGANA" },
	
	// Ѹ쥭ܡɤΤ
	{ KVE_BACKSLASH,	"K_BACKSLASH" },
	
	// ɲå
	{ KVX_RMETA,		"K_RMETA" },
	{ KVX_LMETA,		"K_LMETA" },
	{ KVX_MENU,			"K_MENU" },
};



int cGlobal::P6Model = DEFAULT_MODEL;			//  60:PC-6001 62:PC-6001mk2 66:PC-6601 64:PC-6001mk2SR 68:PC-6601SR
int cGlobal::OverClock = 100;					// СåΨ (1-1000)%
char cGlobal::WCaption[] = "";					// ɥץ

BOOL cGlobal::IsINI = FALSE;					// INIեɹߥե饰
char cGlobal::IniPath[PATH_MAX] = CONF_FILE;	// INIեѥ

char cGlobal::RomPath[]    = "";				// ROMѥ
BOOL cGlobal::UseExtRam    = DEFAULT_EXTRAM;	// ĥRAMȤ TRUE: FALSE:ʤ
char cGlobal::ExtRomPath[] = "";				// ĥROMѥ
char cGlobal::ExtRomFile[] = "";				// ĥROMե̾
BOOL cGlobal::CheckCRC     = TRUE;				// CRCå TRUE:ͭ FALSE:̵

int cGlobal::KeyRepeat = DEFAULT_REPEAT;		// ԡ
BOOL cGlobal::UsePS2KBD = DEFAULT_KEYBOARD;		// PS/2ܡɻȤ? TRUE:PS/2 FALSE:USB
VKeyConv cGlobal::KeyDef[] = {					// ۥ -> P6
	{ KVC_1,			KP6_1 },			// 1	!
	{ KVC_2,			KP6_2 },			// 2	"
	{ KVC_3,			KP6_3 },			// 3	#
	{ KVC_4,			KP6_4 },			// 4	$
	{ KVC_5,			KP6_5 },			// 5	%
	{ KVC_6,			KP6_6 },			// 6	&
	{ KVC_7,			KP6_7 },			// 7	'
	{ KVC_8,			KP6_8 },			// 8	(
	{ KVC_9,			KP6_9 },			// 9	)
	{ KVC_0,			KP6_0 },			// 0
	
	{ KVC_A,			KP6_A },			// a	A
	{ KVC_B,			KP6_B },			// b	B
	{ KVC_C,			KP6_C },			// c	C
	{ KVC_D,			KP6_D },			// d	D
	{ KVC_E,			KP6_E },			// e	E
	{ KVC_F,			KP6_F },			// f	F
	{ KVC_G,			KP6_G },			// g	G
	{ KVC_H,			KP6_H },			// h	H
	{ KVC_I,			KP6_I },			// i	I
	{ KVC_J,			KP6_J },			// j	J
	{ KVC_K,			KP6_K },			// k	K
	{ KVC_L,			KP6_L },			// l	L
	{ KVC_M,			KP6_M },			// m	M
	{ KVC_N,			KP6_N },			// n	N
	{ KVC_O,			KP6_O },			// o	O
	{ KVC_P,			KP6_P },			// p	P
	{ KVC_Q,			KP6_Q },			// q	Q
	{ KVC_R,			KP6_R },			// r	R
	{ KVC_S,			KP6_S },			// s	S
	{ KVC_T,			KP6_T },			// t	T
	{ KVC_U,			KP6_U },			// u	U
	{ KVC_V,			KP6_V },			// v	V
	{ KVC_W,			KP6_W },			// w	W
	{ KVC_X,			KP6_X },			// x	X
	{ KVC_Y,			KP6_Y },			// y	Y
	{ KVC_Z,			KP6_Z },			// z	Z
	
	{ KVC_F1,			KP6_F1 },			// F1
	{ KVC_F2,			KP6_F2 },			// F2
	{ KVC_F3,			KP6_F3 },			// F3
	{ KVC_F4,			KP6_F4 },			// F4
	{ KVC_F5,			KP6_F5 },			// F5
	
	{ KVC_MINUS,		KP6_MINUS },		// -	=
	{ KVC_CARET,		KP6_CARET },		// ^	~
	{ KVC_BACKSPACE,	KP6_YEN },			// BackSpace
	{ KVC_AT,			KP6_AT },			// @	`
	{ KVC_LBRACKET,		KP6_LBRACKET },		// [	{
	{ KVC_SEMICOLON,	KP6_SEMICOLON },	// ;	+
	{ KVC_COLON,		KP6_COLON },		// :	*
	{ KVC_COMMA,		KP6_COMMA },		// ,	<
	{ KVC_PERIOD,		KP6_PERIOD },		// .	>
	{ KVC_SLASH,		KP6_SLASH },		// /	?
	{ KVC_SPACE,		KP6_SPACE },		// Space
	
	{ KVC_ESC,			KP6_ESC },			// ESC
	{ KVC_HANZEN,		KP6_UNKNOWN },		// Ⱦ/
	{ KVC_TAB,			KP6_TAB },			// Tab
	{ KVC_CAPSLOCK,		KP6_UNKNOWN },		// CapsLock
	{ KVC_ENTER,		KP6_RETURN },		// Enter
	{ KVC_LCTRL,		KP6_CTRL },			// L-Ctrl
	{ KVC_RCTRL,		KP6_CTRL },			// R-Ctrl
	{ KVC_LSHIFT,		KP6_SHIFT },		// L-Shift
	{ KVC_RSHIFT,		KP6_SHIFT },		// R-Shift
	{ KVC_LALT,			KP6_GRAPH },		// L-Alt
	{ KVC_RALT,			KP6_GRAPH },		// R-Alt
	{ KVC_PRINT,		KP6_UNKNOWN },		// PrintScreen
	{ KVC_SCROLLLOCK,	KP6_CAPS },			// ScrollLock
	{ KVC_PAUSE,		KP6_KANA },			// Pause
	{ KVC_INSERT,		KP6_INS },			// Insert
	{ KVC_DELETE,		KP6_DEL },			// Delete
	{ KVC_END,			KP6_STOP },			// End
	{ KVC_HOME,			KP6_HOME },			// Home
	{ KVC_PAGEUP,		KP6_PAGE },			// PageUp
	{ KVC_PAGEDOWN,		KP6_MODE },			// PageDown
	
	{ KVC_UP,			KP6_UP },			// 
	{ KVC_DOWN,			KP6_DOWN },			// 
	{ KVC_LEFT,			KP6_LEFT },			// 
	{ KVC_RIGHT,		KP6_RIGHT },		// 
	
	{ KVC_P0,			KP6_P0 },			// [0]
	{ KVC_P1,			KP6_P1 },			// [1]
	{ KVC_P2,			KP6_P2 },			// [2]
	{ KVC_P3,			KP6_P3 },			// [3]
	{ KVC_P4,			KP6_P4 },			// [4]
	{ KVC_P5,			KP6_P5 },			// [5]
	{ KVC_P6,			KP6_P6 },			// [6]
	{ KVC_P7,			KP6_P7 },			// [7]
	{ KVC_P8,			KP6_P8 },			// [8]
	{ KVC_P9,			KP6_P9 },			// [9]
	{ KVC_NUMLOCK,		KP6_UNKNOWN },		// NumLock
	{ KVC_P_PLUS,		KP6_PPLUS },		// [+]
	{ KVC_P_MINUS,		KP6_PMINUS },		// [-]
	{ KVC_P_MULTIPLY,	KP6_PMULTIPLY },	// [*]
	{ KVC_P_DIVIDE,		KP6_PDIVIDE },		// [/]
	{ KVC_P_PERIOD,		KP6_PPERIOD },		// [.]
	{ KVC_P_ENTER,		KP6_PRETURN },		// [Enter]
	
	// ܸ쥭ܡɤΤ
	{ KVC_YEN,			KP6_YEN },			// 	|
	{ KVC_RBRACKET,		KP6_RBRACKET },		// ]	}
	{ KVC_UNDERSCORE,	KP6_UNDERSCORE },	// \	_
//	{ KVC_MUHENKAN,		KP6_UNKNOWN },		// ̵Ѵ
//	{ KVC_HENKAN,		KP6_UNKNOWN },		// Ѵ
//	{ KVC_HIRAGANA,		KP6_UNKNOWN },		// Ҥ餬
	
	// Ѹ쥭ܡɤΤ
	{ KVE_BACKSLASH,	KP6_YEN	 },			// 			BackSlash	|
	
	// ɲå
	{ KVX_RMETA,		KP6_UNKNOWN },		// L-Meta
	{ KVX_LMETA,		KP6_UNKNOWN },		// R-Meta
	{ KVX_MENU,			KP6_UNKNOWN },		// Menu
	
	// Ƽﵡǽ (ΤȤ̵)
//	{ KVC_F6,			KFN_1 },			// F6
//	{ KVC_F7,			KFN_2 },			// F7
//	{ KVC_F8,			KFN_3 },			// F8
//	{ KVC_F9,			KFN_4 },			// F9
//	{ KVC_F10,			KFN_5 },			// F10
//	{ KVC_F11,			KFN_6 },			// F11
//	{ KVC_F12,			KFN_7 },			// F12
	
	{ KVC_LAST,			KP6_LAST }
};


int cGlobal::SampleRate  = DEFAULT_SAMPLE_RATE;	// ץ󥰥졼
int cGlobal::SoundBuffer = SOUND_BUFFER_SIZE;	// ɥХåեĹΨ
int cGlobal::MasterVol   = DEFAULT_MASTERVOL;	// ޥ

int cGlobal::PsgVol      = DEFAULT_PSGVOL;		// PSG

char cGlobal::WavePath[] = "";					// WAVEѥ
int cGlobal::VoiceVol    = DEFAULT_VOICEVOL;	// 

char cGlobal::TapePath[] = "";					// TAPEѥ
char cGlobal::TapeFile[] = "";					// TAPEեեѥ
int cGlobal::CmtVol      = DEFAULT_TAPEVOL;		// TAPE˥
BOOL cGlobal::TurboTAPE  = FALSE;				// Turbo TAPE ͭե饰
BOOL cGlobal::BoostUp    = FALSE;				// Boost Up ͭե饰
int cGlobal::MaxBoost1   = DEFAULT_MAXBOOST60;	// BoostUp Ψ(N60⡼)
int cGlobal::MaxBoost2   = DEFAULT_MAXBOOST62;	// BoostUp Ψ(N60m/N66⡼)

char cGlobal::SaveFile[]  = "";					// TAPE(SAVE)եեѥ

char cGlobal::DiskPath[]  = "";					// DISKѥ
char cGlobal::DiskFile[][PATH_MAX] = { "", "" };	// DISKեեѥ
int cGlobal::FddNum = DEFAULT_FDD;				// FDD³

char cGlobal::ImgPath[]  = "";					// ꡼󥷥åȳǼѥ
int cGlobal::ScrBpp      = DEFAULT_COLOR_MODE;	// 顼⡼ 8:8bit 16:16bit 24:24bit
int cGlobal::Mode4Color  = DEFAULT_MODE4_COLOR;	// ⡼4顼⡼ 0: 1:/ 2:/ 3:ԥ/ 4:/ԥ
BOOL cGlobal::ScanLine   = DEFAULT_SCANLINE;	// 饤 TRUE: FALSE:ʤ
int cGlobal::ScanLineBr  = DEFAULT_SCANLINEBR;	// 饤 (0-100)%
BOOL cGlobal::DispNTSC   = DEFAULT_DISPNTSC;	// 4:3ɽե饰 TRUE:ͭ FALSE:̵
BOOL cGlobal::FullScreen = FALSE;				// ե륹꡼ TRUE:ե FALSE:ɥ
BOOL cGlobal::StatDisp   = TRUE;				// ơСɽ TRUE:ɽ FALSE:ɽ
int cGlobal::FrameSkip   = DEFAULT_FRAMESKIP;	// ե졼ॹå

BOOL cGlobal::AviRle     = TRUE;				// RLEե饰 TRUE:RLEͭ FALSE:٥

char cGlobal::FontPath[]  = "";					// ꡼󥷥åȳǼѥ
char cGlobal::FontFileZ[] = "";					// ѥեȥեѥ
char cGlobal::FontFileH[] = "";					// Ⱦѥեȥեѥ

char cGlobal::PrinterFile[] = "";				// ץ󥿥եեѥ

BOOL cGlobal::CkQuit     = FALSE;				// λǧ TRUE: FALSE:ʤ

char cGlobal::DokoFile[] = DOKODEMO_FILE;		// ɤǤSAVEեեѥ

SDL_Color cGlobal::UserColor[128];				// 桼ꥫ顼ǡ ( R,G,B,0  0-255 )
const SDL_Color cGlobal::STDColor[] = {			// ɸ५顼ǡ ( R,G,B,0  0-255 )
				// ƥ५顼
				{   0,   0,   0, 0 },	// 00:
				{   0,   0, 128, 0 },	// 01:
				{   0, 128,   0, 0 },	// 02:
				{   0, 128, 128, 0 },	// 03:
				{ 128,   0,   0, 0 },	// 04:
				{ 128,   0, 128, 0 },	// 05:
				{ 128, 128,   0, 0 },	// 06:
				{ 128, 128, 128, 0 },	// 07:
				{ 192, 192, 192, 0 },	// 08:
				{   0,   0, 255, 0 },	// 09:
				{   0, 255,   0, 0 },	// 10:
				{   0, 255, 255, 0 },	// 11:
				{ 255,   0,   0, 0 },	// 12:
				{ 255,   0, 255, 0 },	// 13:
				{ 255, 255,   0, 0 },	// 14:
				{ 255, 255, 255, 0 },	// 15:
				
				{  20,  20,  20, 0 },	// 16:(mode 1,2 ܡ)
				
				// mode 1
				{ 172, 232, 120, 0 },	// 17:(Set1)
				{  40, 152,  20, 0 },	// 18:
				{ 252, 224, 208, 0 },	// 19:(Set2)
				{ 252, 112,   0, 0 },	// 20:
				// mode 2
				{   0, 128,   0, 0 },	// 21:
				{ 255, 255,   0, 0 },	// 22:
				{   0,   0, 255, 0 },	// 23:
				{ 255,   0,   0, 0 },	// 24:
				{ 255, 255, 255, 0 },	// 25:
				{   0, 255, 255, 0 },	// 26:
				{ 255,   0, 255, 0 },	// 27:ޥ
				{ 255, 112,   0, 0 },	// 28:
				// mode 3
				{  40, 255, 100, 0 },	// 29:
				{ 255, 255,   0, 0 },	// 30:
				{   0,   0, 255, 0 },	// 31:
				{ 255,   0,   0, 0 },	// 32:
				{ 255, 255, 255, 0 },	// 33:
				{   0, 255, 255, 0 },	// 34:
				{ 255,   0, 255, 0 },	// 35:ޥ
				{ 255, 112,   0, 0 },	// 36:
				// mode 4
				{  20,  40,  20, 0 },	// 37:(Set1)
				{  60, 220,  60, 0 },	// 38:
				{  40,  20,  20, 0 },	// 39:(Set2)
				{ 224, 255, 208, 0 },	// 40:
				
				{ 225,  47,  32, 0 },	// 41:ˤ (Set1)
				{ 242, 231,   9, 0 },	// 42:ˤ (Set1)
				{   0,   0,   0, 0 },	// 43:ˤ - (Set1)
				{   0,   0,   0, 0 },	// 44:ˤ - (Set1)
				{ 255,  48,   0, 0 },	// 45:ˤ (Set2)
				{ 120, 220, 255, 0 },	// 46:ˤ (Set2)
				{ 255,  72,  72, 0 },	// 47:ˤ (Set2)
				{  40, 255,  80, 0 },	// 48:ˤ (Set2)
				
				// mk2
				{  20,  20,  20, 0 },	// 49:Ʃ()
				{ 255, 172,   0, 0 },	// 50:
				{   0, 255, 172, 0 },	// 51:
				{ 172, 255,   0, 0 },	// 52:
				{ 172,   0, 255, 0 },	// 53:Ļ
				{ 255,   0, 172, 0 },	// 54:ֻ
				{   0, 172, 255, 0 },	// 55:
				{ 172, 172, 172, 0 },	// 56:
				{  20,  20,  20, 0 },	// 57:
				{ 255,   0,   0, 0 },	// 58:
				{   0, 255,   0, 0 },	// 59:
				{ 255, 255,   0, 0 },	// 60:
				{   0,   0, 255, 0 },	// 61:
				{ 255,   0, 255, 0 },	// 62:ޥ
				{   0, 255, 255, 0 },	// 63:
				{ 255, 255, 255, 0 },	// 64:
				
				// mode 4 ˤ
				{   0,   0,   0, 0 },	// 65:äݤ
				{   0,   0,   0, 0 },	// 66:Ĥäݤ
				{   0,   0,   0, 0 },	// 67:֤äݤ
				{ 255, 255, 255, 0 },	// 68:äݤ
				{ 255, 255, 255, 0 },	// 69:Ĥäݤ
				{ 255, 255, 255, 0 },	// 70:֤äݤ
				{ 120, 220, 255, 0 },	// 71:äݤ
				{ 120, 220, 255, 0 },	// 72:äݤ
				{ 120, 220, 255, 0 },	// 73:֤äݤ
				{ 255,  48, 255, 0 },	// 74:äݤ
				{ 255,  48, 255, 0 },	// 75:äݤ
				{ 255,  48, 255, 0 }	// 76:Ĥäݤ
			};


#ifdef USEMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
BOOL cGlobal::MonDisp = FALSE;					// ˥ɥɽ TRUE:ɽ FALSE:ɽ
BOOL cGlobal::TimerIntr = TRUE;					// ޳ߵĥե饰 TRUE: FALSE:Ե
#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //



////////////////////////////////////////////////////////////////
// 󥹥ȥ饯
////////////////////////////////////////////////////////////////
cGlobal::cGlobal( void ){}


////////////////////////////////////////////////////////////////
// ǥȥ饯
////////////////////////////////////////////////////////////////
cGlobal::~cGlobal( void ){}


////////////////////////////////////////////////////////////////
// (INIեɹ)
//
// :	ʤ
// :	BOOL	TRUE: FALSE:
////////////////////////////////////////////////////////////////
BOOL cGlobal::LoadIniFile( void )
{
	// INIեɹ
	if( !IsINI ){
		cIni *Ini = NULL;
		
		// INIեΥѥ
		sprintf( IniPath, "%s" CONF_FILE, OSD_GetModulePath() );
		
		try{
			// INIե뤬ʤʤǥեȤǺ
			if( !OSD_FileExist( IniPath ) )
				if( !CreateINI() ) throw Error::IniDefault;
			
			// INIե򳫤
			Ini = new cIni();
			if( !Ini->Init( IniPath ) ) throw Error::IniDefault;
		}
		catch( std::bad_alloc ){	// new ˼Ԥ
			Error::SetError( Error::MemAllocFailed );
			return FALSE;
		}
		catch( Error::Errno i ){	// 㳰ȯ
			Error::SetError( i );
			
			if( Ini ) delete Ini;
			return FALSE;
		}
		
		IsINI = TRUE;
		
		char str[PATH_MAX];
		int st;
		
		//  ----------
		// 
		Ini->GetInt( "CONFIG", "Model", &P6Model, DEFAULT_MODEL );
		switch( P6Model ){	// ɥץ
		case 62: sprintf( WCaption, "PC6001V (PC-6001mk2) Ver."   VERSION ); break;
		case 66: sprintf( WCaption, "PC6001V (PC-6601) Ver."      VERSION ); break;
		case 64: sprintf( WCaption, "PC6001V (PC-6001mk2SR) Ver." VERSION ); break;
		case 68: sprintf( WCaption, "PC6001V (PC-6601SR) Ver."    VERSION ); break;
		default: sprintf( WCaption, "PC6001V (PC-6001) Ver."      VERSION ); break;
		}
		
		// СåΨ
		Ini->GetInt( "CONFIG", "OverClock", &OverClock, 100 );
		
		
		// ط ----------
		// ĥRAMȤ
		Ini->GetTruth( "CONFIG", "ExtRam", &st, DEFAULT_EXTRAM );
		UseExtRam = st ? TRUE : FALSE;
		
		// CRCå
		Ini->GetTruth( "CONFIG", "CheckCRC", &st, TRUE );
		CheckCRC = st ? TRUE : FALSE;
		
		
		// ɴط ----------
		// ץ󥰥졼
		Ini->GetInt( "SOUND", "SampleRate",   &SampleRate,  DEFAULT_SAMPLE_RATE );
		
		// ɥХåեĹΨ
		Ini->GetInt( "SOUND", "SoundBuffer",  &SoundBuffer, SOUND_BUFFER_SIZE );
		
		// ޥ
		Ini->GetInt( "SOUND", "MasterVolume", &MasterVol,   DEFAULT_MASTERVOL );
		
		// PSG
		Ini->GetInt( "SOUND", "PsgVolume",    &PsgVol,      DEFAULT_PSGVOL );
		
		// 
		Ini->GetInt( "SOUND", "VoiceVolume",  &VoiceVol,    DEFAULT_VOICEVOL );
		
		// TAPE˥
		Ini->GetInt( "SOUND", "TapeVolume",   &CmtVol,      DEFAULT_TAPEVOL );
		
		
		// CMTط ----------
		// Turbo TAPE ͭե饰
		Ini->GetTruth( "CONFIG", "TurboTAPE", &TurboTAPE, DEFAULT_TURBO );
		
		// Boost Up ͭե饰
		Ini->GetTruth( "CONFIG", "BoostUp", &BoostUp, DEFAULT_BOOST );
		
		// BoostUp Ψ(N60⡼)
		Ini->GetInt( "CONFIG", "MaxBoost60", &MaxBoost1, DEFAULT_MAXBOOST60 );
		
		// BoostUp Ψ(N60m/N66⡼)
		Ini->GetInt( "CONFIG", "MaxBoost62", &MaxBoost2, DEFAULT_MAXBOOST62 );
		
		
		// DISKط ----------
		// FDD³
		Ini->GetInt( "CONFIG", "FDD", &FddNum, DEFAULT_FDD );
		if( FddNum > 2 ) FddNum = 2;	// 2ޤ
		
		
		// ̴ط ----------
		// 顼⡼
		Ini->GetInt( "DISPLAY", "ScrBpp", &ScrBpp, 8 );
		
		// ⡼4顼
		Ini->GetInt( "DISPLAY", "Mode4Color", &Mode4Color, DEFAULT_MODE4_COLOR );
		
		// 饤
		Ini->GetTruth( "DISPLAY", "ScanLine", &ScanLine, DEFAULT_SCANLINE );
		
		// 饤
		Ini->GetInt( "DISPLAY", "ScanLineBr", &ScanLineBr, DEFAULT_SCANLINEBR );
		
		// 4:3ɽ
		Ini->GetTruth( "DISPLAY", "DispNTSC", &DispNTSC, DEFAULT_DISPNTSC );
		
		// ե졼ॹå
		Ini->GetInt( "DISPLAY", "FrameSkip", &FrameSkip, DEFAULT_FRAMESKIP );
		
		
		// ӥǥץط ----------
		// RLEե饰
		Ini->GetTruth( "MOVIE", "AviRle", &AviRle, TRUE );
		
		
		// եط ----------
		// ƥեѥȤư
		// ĥROMե̾(ư˼ưޥ)
		Ini->GetString( "FILES", "ExtRom", str, "" );
		Delimiter( str );
		if( OSD_FileExist( str ) ) strncpy( ExtRomFile, str, PATH_MAX );
		
		// TAPEե̾(ư˼ưޥ)
		Ini->GetString( "FILES", "tape", str, "" );
		Delimiter( str );
		if( OSD_FileExist( str ) ) strncpy( TapeFile, str, PATH_MAX );
		
		// TAPE(SAVE)ե̾(SAVE˼ưޥ)
		sprintf( str, "%s%s", TapePath, SAVE_FILE );	// ǥե
		Ini->GetString( "FILES", "save", str, str );
		Delimiter( str );
		strncpy( SaveFile, str, PATH_MAX );
		
		// DISKե̾(ư˼ưޥ)
		Ini->GetString( "FILES", "disk", str, "" );
		Delimiter( str );
		if( OSD_FileExist( str ) ) strncpy( DiskFile[0], str, PATH_MAX );
		
		// ץ󥿥ե̾
		sprintf( str, "%s" PRINTER_FILE, OSD_GetModulePath() );	// ǥե
		Ini->GetString( "FILES", "printer", str, str );
		Delimiter( str );
		strncpy( PrinterFile, str, PATH_MAX );
		
		// ѥեȥե̾
		sprintf( str, "%s%s", FontPath, FONTZ_FILE );	// ǥե
		Ini->GetString( "FILES", "fontz", str, str );
		Delimiter( str );
		strncpy( FontFileZ, str, PATH_MAX );
		
		// Ⱦѥեȥե̾
		sprintf( str, "%s%s", FontPath, FONTH_FILE );	// ǥե
		Ini->GetString( "FILES", "fonth", str, str );
		Delimiter( str );
		strncpy( FontFileH, str, PATH_MAX );
		
		
		// ѥط ----------
		// ROMѥ
		sprintf( str, "%s" ROM_DIR, OSD_GetModulePath() );		// ǥե
		Ini->GetString( "PATH", "RomPath", str, str );
		Delimiter( str );
		AddDelimiter( str );
		strncpy( RomPath, str, PATH_MAX );
		
		// ĥROMѥ
		sprintf( str, "%s" EXTROM_DIR, OSD_GetModulePath() );	// ǥե
		Ini->GetString( "PATH", "ExtRomPath", str, str );
		Delimiter( str );
		AddDelimiter( str );
		strncpy( ExtRomPath, str, PATH_MAX );
		
		// WAVEѥ
		sprintf( str, "%s" WAVE_DIR, OSD_GetModulePath() );		// ǥե
		Ini->GetString( "PATH", "WavePath", str, str );
		Delimiter( str );
		AddDelimiter( str );
		strncpy( WavePath, str, PATH_MAX );
		
		// TAPEѥ
		sprintf( str, "%s" TAPE_DIR, OSD_GetModulePath() );		// ǥե
		Ini->GetString( "PATH", "TapePath", str, str );
		Delimiter( str );
		AddDelimiter( str );
		strncpy( TapePath, str, PATH_MAX );
		
		// DISKѥ
		sprintf( str, "%s" DISK_DIR, OSD_GetModulePath() );		// ǥե
		Ini->GetString( "PATH", "DiskPath", str, str );
		Delimiter( str );
		AddDelimiter( str );
		strncpy( DiskPath, str, PATH_MAX );
		
		// IMGѥ
		sprintf( str, "%s" IMAGE_DIR, OSD_GetModulePath() );	// ǥե
		Ini->GetString( "PATH", "ImgPath", str, str );
		Delimiter( str );
		AddDelimiter( str );
		strncpy( ImgPath, str, PATH_MAX );
		
		// FONTѥ
		sprintf( str, "%s" FONT_DIR, OSD_GetModulePath() );		// ǥե
		Ini->GetString( "PATH", "FontPath", str, str );
		Delimiter( str );
		AddDelimiter( str );
		strncpy( FontPath, str, PATH_MAX );
		
		
		// ǧط ----------
		// λ
		Ini->GetTruth( "CHECK", "CkQuit", &CkQuit, FALSE );
		
		
		// ѥå ----------
		for( int i=0; i<(int)(sizeof(STDColor)/sizeof(SDL_Color)); i++ ){
			char strin[16];
			sprintf( strin, "COL%03d", i );
			sprintf( str,   "%02X%02X%02X", STDColor[i].r, STDColor[i].g, STDColor[i].b );
			if( Ini->GetString( "COLOR", strin, str, str ) ){
				st = strtol( str, NULL, 16 );
				UserColor[i].r = (st>>16)&0xff;
				UserColor[i].g = (st>> 8)&0xff;
				UserColor[i].b = (st    )&0xff;
			}
		}
		
		
		// ܡɴط ----------
		// ԡ
		Ini->GetInt( "KEY", "KeyRepeat", &KeyRepeat, DEFAULT_REPEAT );
		// PS/2ܡɻȤ?
		Ini->GetTruth( "KEY", "UsePS2KBD", &UsePS2KBD, DEFAULT_KEYBOARD );
		
		// 
		int i = 0;
		while( KeyDef[i].PCKey != KVC_LAST ){
			// ɤ̾μ
			char *k1 = GetPCKeyName( KeyDef[i].PCKey );
			char *k2 = GetP6KeyName( KeyDef[i].P6Key );
			
			if( Ini->GetString( "KEY", k1, str, k2 ) )
				KeyDef[i].P6Key = GetP6KeyCode( str );
			
			i++;
		}
		
		
		// INIեĤ
		delete Ini;
	}
	
	// ɤǤSAVEե̾
	sprintf( DokoFile, "%s" DOKODEMO_FILE, OSD_GetModulePath() );	// ǥե
	
	MonDisp   = FALSE;		// ˥ɥɽ
	TimerIntr = TRUE;		// ޳ߵ
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
// INIե뿷
//
// :	ʤ
// :	BOOL	TRUE: FALSE:
////////////////////////////////////////////////////////////////
BOOL cGlobal::CreateINI( void )
{
	cIni *Ini = NULL;
	FILE *fp;
	try{
		fp = FOPENEN( IniPath, "wt" );
		if( !fp ) throw Error::IniWriteFailed;
		// ȥԤϤưöĤ
		fprintf( fp, MSINI_TITLE );
		fclose( fp );
		
		// INIե򳫤
		Ini = new cIni();
		if( !Ini->Init( IniPath ) ) throw Error::IniDefault;
	}
	catch( std::bad_alloc ){	// new ˼Ԥ
		Error::SetError( Error::MemAllocFailed );
		return FALSE;
	}
	catch( Error::Errno i ){	// 㳰ȯ
		Error::SetError( i );
		
		switch(i){
		case Error::IniWriteFailed:	// INIեν񤭹ߤ˼Ԥ
			return FALSE;
			
		case Error::IniDefault:		// INIեν˼Ԥ
		default:				// 
			delete Ini;
		}
		return FALSE;
	}
	
	// INI֥Ƚ
	InitIni( Ini, TRUE );
	
	// INIե˽
	Ini->Write();
	
	delete Ini;
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
// INI֥Ƚ
//
// :	over	TRUE: FALSE:Ρɤ¸ߤƤѥ
// :	ʤ
////////////////////////////////////////////////////////////////
void cGlobal::InitIni( cIni *Ini, BOOL over )
{
	if( !Ini ) return;
	
	char str[PATH_MAX];
	
	// [CONFIG]
	// 
	if( over || !Ini->GetString( "CONFIG", "Model", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_Model,		"Model",		"%02d",	DEFAULT_MODEL );
	
	// FDD
	if( over || !Ini->GetString( "CONFIG", "FDD", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_FDD,			"FDD",			"%d",	DEFAULT_FDD );
	
	// ĥRAM
	if( over || !Ini->GetString( "CONFIG", "ExtRam", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_ExtRam,		"ExtRam",		"%s",	DEFAULT_EXTRAM ? "Yes" : "No" );
	
	// Turbo TAPE
	if( over || !Ini->GetString( "CONFIG", "TurboTAPE", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_TurboTAPE,	"TurboTAPE",	"%s",	DEFAULT_TURBO ? "Yes" : "No" );
	
	// Boost Up
	if( over || !Ini->GetString( "CONFIG", "BoostUp", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_BoostUp,		"BoostUp",		"%s",	DEFAULT_BOOST ? "Yes" : "No" );
	
	// BoostUp Ψ(N60⡼)
	if( over || !Ini->GetString( "CONFIG", "MaxBoost60", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_MaxBoost60,	"MaxBoost60",	"%d",	DEFAULT_MAXBOOST60 );
	
	// BoostUp Ψ(N60m/N66⡼)
	if( over || !Ini->GetString( "CONFIG", "MaxBoost62", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_MaxBoost62,	"MaxBoost62",	"%d",	DEFAULT_MAXBOOST62 );
	
	// СåΨ
	if( over || !Ini->GetString( "CONFIG", "OverClock", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_OverClock,	"OverClock",	"%d",	100 );
	
	// CRCå
	if( over || !Ini->GetString( "CONFIG", "CheckCRC", str, str ) )
		Ini->PutEntry( "CONFIG", MSINI_CheckCRC,	"CheckCRC",		"%s",	"Yes" );
	
	// [DISPLAY]
	// 顼⡼
	if( over || !Ini->GetString( "DISPLAY", "ScrBpp", str, str ) )
		Ini->PutEntry( "DISPLAY", MSINI_ScrBpp,		"ScrBpp",		"%d",	DEFAULT_COLOR_MODE );
	
	// MODE4顼
	if( over || !Ini->GetString( "DISPLAY", "Mode4Color", str, str ) )
		Ini->PutEntry( "DISPLAY", MSINI_Mode4Color,	"Mode4Color",	"%d",	DEFAULT_MODE4_COLOR );
	
	// 饤
	if( over || !Ini->GetString( "DISPLAY", "ScanLine", str, str ) )
		Ini->PutEntry( "DISPLAY", MSINI_ScanLine,	"ScanLine",		"%s",	DEFAULT_SCANLINE ? "Yes" : "No" );
	
	// 饤
	if( over || !Ini->GetString( "DISPLAY", "ScanLineBr", str, str ) )
		Ini->PutEntry( "DISPLAY", MSINI_ScanLineBr,	"ScanLineBr",	"%d",	DEFAULT_SCANLINEBR );
	
	// 4:3ɽ
	if( over || !Ini->GetString( "DISPLAY", "DispNTSC", str, str ) )
		Ini->PutEntry( "DISPLAY", MSINI_DispNTSC,	"DispNTSC",		"%s",	DEFAULT_DISPNTSC ? "Yes" : "No" );
	
	// ե졼ॹå
	if( over || !Ini->GetString( "DISPLAY", "FrameSkip", str, str ) )
		Ini->PutEntry( "DISPLAY", MSINI_FrameSkip,	"FrameSkip",	"%d",	DEFAULT_FRAMESKIP );
	
	
	// [SOUND]
	// ץ󥰥졼
	if( over || !Ini->GetString( "SOUND", "SampleRate", str, str ) )
		Ini->PutEntry( "SOUND", MSINI_SampleRate,	"SampleRate",	"%d",	DEFAULT_SAMPLE_RATE );
	
	// ɥХåեĹΨ
	if( over || !Ini->GetString( "SOUND", "SoundBuffer", str, str ) )
		Ini->PutEntry( "SOUND", MSINI_SoundBuffer,	"SoundBuffer",	"%d",	SOUND_BUFFER_SIZE );
	
	// ޥ
	if( over || !Ini->GetString( "SOUND", "MasterVolume", str, str ) )
		Ini->PutEntry( "SOUND", MSINI_MasterVolume,	"MasterVolume",	"%d",	DEFAULT_MASTERVOL );
	
	// PSG
	if( over || !Ini->GetString( "SOUND", "PsgVolume", str, str ) )
		Ini->PutEntry( "SOUND", MSINI_PsgVolume,	"PsgVolume",	"%d",	DEFAULT_PSGVOL );
	
	// 
	if( over || !Ini->GetString( "SOUND", "VoiceVolume", str, str ) )
		Ini->PutEntry( "SOUND", MSINI_VoiceVolume,	"VoiceVolume",	"%d",	DEFAULT_VOICEVOL );
	
	// TAPE˥
	if( over || !Ini->GetString( "SOUND", "TapeVolume", str, str ) )
		Ini->PutEntry( "SOUND", MSINI_TapeVolume,	"TapeVolume",	"%d",	DEFAULT_TAPEVOL );
	
	
	// [MOVIE]
	// RLEե饰
	if( over || !Ini->GetString( "MOVIE", "AviRle", str, str ) )
		Ini->PutEntry( "MOVIE", MSINI_AviRle,	"AviRle",		"%s",	"Yes" );
	
	
	//[FILES]
	// ĥROMե̾(ư˼ưޥ)
	if( over || !Ini->GetString( "FILES", "ExtRom", str, str ) )
		Ini->PutEntry( "FILES",	MSINI_ExtRom,	"ExtRom",	"" );
	
	// TAPEե̾(ư˼ưޥ)
	if( over || !Ini->GetString( "FILES", "tape", str, str ) )
		Ini->PutEntry( "FILES", MSINI_tape,		"tape",		"" );
	
	// TAPE(SAVE)ե̾(SAVE˼ưޥ)
	if( over || !Ini->GetString( "FILES", "save", str, str ) ){
		sprintf( str, "%s%s/%s", OSD_GetModulePath(), TAPE_DIR, SAVE_FILE );
		UnDelimiter( str );
		Ini->PutEntry( "FILES",	MSINI_save,		"save",		str );
	}
	
	// DISKե̾(ư˼ưޥ)
	if( over || !Ini->GetString( "FILES", "disk", str, str ) )
		Ini->PutEntry( "FILES",	MSINI_disk,		"disk",		"" );
	
	// ץ󥿥ե̾
	if( over || !Ini->GetString( "FILES", "printer", str, str ) ){
		sprintf( str, "%s" PRINTER_FILE, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "FILES",	MSINI_printer,	"printer",	str );
	}
	
	// ѥեȥե̾
	if( over || !Ini->GetString( "FILES", "fontz", str, str ) ){
		sprintf( str, "%s%s/%s", OSD_GetModulePath(), FONT_DIR, FONTZ_FILE );
		UnDelimiter( str );
		Ini->PutEntry( "FILES",	MSINI_fontz,	"fontz",	str );
	}
	
	// Ⱦѥեȥե̾
	if( over || !Ini->GetString( "FILES", "fonth", str, str ) ){
		sprintf( str, "%s%s/%s", OSD_GetModulePath(), FONT_DIR, FONTH_FILE );
		UnDelimiter( str );
		Ini->PutEntry( "FILES",	MSINI_fonth,	"fonth",	str );
	}
	
	// [PATH]
	// ROMѥ
	if( over || !Ini->GetString( "PATH", "RomPath", str, str ) ){
		sprintf( str, "%s" ROM_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "PATH",	MSINI_RomPath,	"RomPath",	str );
	}
	
	// TAPEѥ
	if( over || !Ini->GetString( "PATH", "TapePath", str, str ) ){
		sprintf( str, "%s" TAPE_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "PATH",	MSINI_TapePath,	"TapePath",	str );
	}
	
	// DISKѥ
	if( over || !Ini->GetString( "PATH", "DiskPath", str, str ) ){
		sprintf( str, "%s" DISK_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "PATH",	MSINI_DiskPath,	"DiskPath",	str );
	}
	
	// ĥROMѥ
	if( over || !Ini->GetString( "PATH", "ExtRomPath", str, str ) ){
		sprintf( str, "%s" EXTROM_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "PATH",	MSINI_ExtRomPath,	"ExtRomPath",	str );
	}
	
	// WAVEѥ
	if( over || !Ini->GetString( "PATH", "WavePath", str, str ) ){
		sprintf( str, "%s" WAVE_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "PATH",	MSINI_WavePath,	"WavePath",	str );
	}
	
	// IMGѥ
	if( over || !Ini->GetString( "PATH", "ImgPath", str, str ) ){
		sprintf( str, "%s" IMAGE_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "PATH",	MSINI_ImgPath,	"ImgPath",	str );
	}
	
	// FONTѥ
	if( over || !Ini->GetString( "PATH", "FontPath", str, str ) ){
		sprintf( str, "%s" FONT_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		Ini->PutEntry( "PATH",	MSINI_FontPath,	"FontPath",	str );
	}
	
	// [CHECK]
	if( over || !Ini->GetString( "CHECK", "CkQuit", str, str ) )
		Ini->PutEntry( "CHECK", MSINI_CkQuit,	"CkQuit",		"%s",	"No" );
	
	
	// [COLOR]
	// ѥå
	for( int i=16; i<(int)(sizeof(STDColor)/sizeof(SDL_Color)); i++ ){
		char stren[16];
		sprintf( stren, "COL%03d", i );
		if( over || !Ini->GetString( "COLOR", stren, str, str ) )
			Ini->PutEntry( "COLOR", OSD_ColorName( i-16 ), stren, "%02X%02X%02X", STDColor[i].r, STDColor[i].g, STDColor[i].b );
	}
	
	// [KEY]
	// ԡ
	if( over || !Ini->GetString( "KEY", "KeyRepeat", str, str ) )
		Ini->PutEntry( "KEY", MSINI_KeyRepeat,	"KeyRepeat",	"%d",	DEFAULT_REPEAT );
	
	// PS/2ܡɻȤ?
	if( over || !Ini->GetString( "KEY", "UsePS2KBD", str, str ) )
		Ini->PutEntry( "KEY", MSINI_UsePS2KBD,	"UsePS2KBD",		"%s",	"No" );
	
	// 
	int i = 0;
	while( KeyDef[i].PCKey != KVC_LAST ){
		
		// ɤ̾μ
		char *k1 = GetPCKeyName( KeyDef[i].PCKey );
		char *k2 = GetP6KeyName( KeyDef[i].P6Key );
		
		if( k1 && k2 ){
			if( over || !Ini->GetString( "KEY", k1, str, str ) )
				Ini->PutEntry( "KEY", OSD_KeyName( KeyDef[i].PCKey ), k1, k2 );
		}
		
		i++;
	}
	
}


////////////////////////////////////////////////////////////////
// ɤǤSAVE
//
// :	Ini		INI֥ȥݥ
// :	BOOL	TRUE: FALSE:
////////////////////////////////////////////////////////////////
BOOL cGlobal::DokoSave( cIni *Ini )
{
	if( !Ini ) return FALSE;
	
	// 
	Ini->PutEntry( "GLOBAL", NULL, "Version",	VERSION );
	Ini->PutEntry( "GLOBAL", NULL, "P6Model",	"%02d",	P6Model );
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
// ɤǤLOAD
//
// :	Ini		INI֥ȥݥ
// :	BOOL	TRUE: FALSE:
////////////////////////////////////////////////////////////////
BOOL cGlobal::DokoLoad( cIni *Ini )
{
	int st;
	char strva[256];
	
	if( !Ini ) return FALSE;
	
	// 
	Ini->GetString( "GLOBAL", "Version", strva, "" );
	if( strcmp( strva, VERSION ) ){
		Error::SetError( Error::DokoDiffVersion );
		return FALSE;
	}
	Ini->GetInt( "GLOBAL", "P6Model",	&st, 0 );
	if( st == P6Model )
		P6Model = st;
	else{
		Error::SetError( Error::DokoDiffModel );
		return FALSE;
	}
	
	return TRUE;
}


////////////////////////////////////////////////////////////////
// ۥɤ̾μ
//
// :	sym		ۥ
// :	char *	̾ʸؤΥݥ(ĤʤNULL)
////////////////////////////////////////////////////////////////
char *cGlobal::GetPCKeyName( PCKEYsym sym )
{
	char *str = NULL;
	for( int i=0; i<(int)(sizeof(PCKeyNameDef)/sizeof(PCKeyName)); i++ ){
		if( PCKeyNameDef[i].PCKey == sym ){
			str = PCKeyNameDef[i].Name;
			break;
		}
	}
	return str;
}


////////////////////////////////////////////////////////////////
// P6ɤ̾μ
//
// :	sym		P6
// :	char *	̾ʸؤΥݥ(ĤʤNULL)
////////////////////////////////////////////////////////////////
char *cGlobal::GetP6KeyName( P6KEYsym sym )
{
	char *str = NULL;
	for( int i=0; i<(int)(sizeof(P6KeyNameDef)/sizeof(P6KeyName)); i++ ){
		if( P6KeyNameDef[i].P6Key == sym ){
			str = P6KeyNameDef[i].Name;
			break;
		}
	}
	return str;
}


////////////////////////////////////////////////////////////////
// ̾Τ鲾ۥɤ
//
// :	str			̾ʸؤΥݥ
// :	PCKEYsym	ۥ
////////////////////////////////////////////////////////////////
PCKEYsym cGlobal::GetPCKeyCode( char *str )
{
	PCKEYsym sym = KVC_UNKNOWN;
	for( int i=0; i<(int)(sizeof(PCKeyNameDef)/sizeof(PCKeyName)); i++ ){
		if( !stricmp( PCKeyNameDef[i].Name, str ) ){
			sym = PCKeyNameDef[i].PCKey;
			break;
		}
	}
	return sym;
}


////////////////////////////////////////////////////////////////
// ̾ΤP6ɤ
//
// :	str			̾ʸؤΥݥ
// :	P6KEYsym	P6
////////////////////////////////////////////////////////////////
P6KEYsym cGlobal::GetP6KeyCode( char *str )
{
	P6KEYsym sym = KP6_UNKNOWN;
	for( int i=0; i<(int)(sizeof(P6KeyNameDef)/sizeof(P6KeyName)); i++ ){
		if( !stricmp( P6KeyNameDef[i].Name, str ) ){
			sym = P6KeyNameDef[i].P6Key;
			break;
		}
	}
	return sym;
}


