#ifndef MEMORY_H_INCLUDED
#define MEMORY_H_INCLUDED

#include "pc6001v.h"
#include "device.h"
#include "ini.h"


// ֥å
#define MAXRMB	(16)
#define MAXWMB	(8)

////////////////////////////////////////////////////////////////
// 饹
////////////////////////////////////////////////////////////////
// ֥å١饹
class MemBlock {
protected:
	char Name[33];				// ֥å̾
	BYTE *MB;					// ֥åؤΥݥ
	int RWait;					// (ɹ)
	int WWait;					// ()
	
public:
	MemBlock();									// 󥹥ȥ饯
	virtual ~MemBlock();						// ǥȥ饯
	
	void SetMemory( char *, BYTE *, int, int );	// ֥å
	void SetWait( int, int );					// 
	int GetWait();								// ȼ
	
	char *GetName(){ return Name; }				// ֥å̾
	
	BYTE Read( WORD, int * = NULL );			// ꡼
	virtual void Write( WORD, BYTE, int * = NULL ){}	// 饤
};

// ROM饹
class RomBlock : public MemBlock {
public:
	RomBlock();									// 󥹥ȥ饯
	~RomBlock();								// ǥȥ饯
	
	void Write( WORD, BYTE, int * = NULL );		// 饤
};

// RAM饹
class RamBlock : public MemBlock {
public:
	RamBlock();									// 󥹥ȥ饯
	~RamBlock();								// ǥȥ饯
	
	void Write( WORD, BYTE, int * = NULL );		// 饤
};


class MEM6 : public IDoko {
protected:
	// ¤
	typedef struct{
		char *FileName;	// ե̾
		DWORD Size;		// 
		DWORD Crc;		// CRC32
		BYTE Init;		// ǡ
		int WaitR;		// (Read)
		int WaitW;		// (Write)
	} MEMINFO;
	
	// 
	static const MEMINFO IEXTROM;
	static const MEMINFO IEMPTROM;
	static const MEMINFO IEMPTRAM;
	
	BOOL CGBank;				// CG ROM BANK TRUE:ͭ FALSE:̵
	BOOL UseExtRom;				// ĥROM     TRUE:ͭ FALSE:̵
	
	BYTE *MainRom;				// BASIC ROM	60:16KB 62,66:32KB
	BYTE *ExtRom;				// ĥ ROM		60:16KB 62,66:16KB
	BYTE *CGRom1;				// CG ROM1		60: 4KB 62,66: 8KB
	BYTE *MainRam;				// RAM			60:32KB 62,66:64KB
	BYTE *EmptyRom;				// ̤ROM	60: 8KB 62,66: 8KB
	BYTE *EmptyRam;				// ̤RAM	60: 8KB 62,66: 8KB
	
	RomBlock RomB[MAXRMB];		// ROM֥å
	RamBlock RamB[MAXWMB];		// RAM֥å
	
	MemBlock *Rm_blk[8];		// ꡼ɻ֥åݥ(8KB*8)
	MemBlock *Wm_blk[8];		// 饤Ȼ֥åݥ(8KB*8)
	
	BYTE ram_sel[4];			// RAM,RAM (16KB*4)
	
	char FilePath[PATH_MAX];	// ĥROMեեѥ
	int M1Wait;					// M1
	BOOL EnableChkCRC;			// CRCåͭ?
	
	BOOL AllocMemory( BYTE **, const MEMINFO *, char * );	// ݤROMեɹ
	BOOL InitCommon( BOOL );				// ʬ
	
public:
	MEM6();									// 󥹥ȥ饯
	virtual ~MEM6();						// ǥȥ饯
	
	virtual BOOL Init( char *, BOOL ) = 0;	// 
	virtual void Reset(){};					// ꥻå
	
	BYTE Fetch( WORD, int * = NULL );		// եå(M1)
	BYTE Read( WORD, int * = NULL );		// ꡼
	void Write( WORD, BYTE, int * = NULL );	// 饤
	
	BOOL MountExtRom( char * );				// ĥROM ޥ
	void UnmountExtRom();					// ĥROM ޥ
	char *GetFile();						// ĥROMեѥ
	
	// 8255ϴϢؿ
	virtual void SetCGBank( BOOL );			// CG ROM BANK ڤؤ
	
	// ľܥؿ
	virtual BYTE ReadMainRom( WORD addr ){ return MainRom[addr&0x3fff]; }
	virtual BYTE ReadMainRam( WORD addr ){ return MainRam[addr&0x3fff]; }
	virtual BYTE ReadExtRom ( WORD addr ){ return ExtRom ? ExtRom[addr&0x3fff] : 0xff; }
	virtual BYTE ReadCGrom1 ( WORD addr ){ return CGRom1[addr&0x1fff]; }
	virtual BYTE ReadCGrom2 ( WORD )     { return 0xff; }
	
	#ifdef USEMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	virtual char *GetReadMemBlk( int );
	virtual char *GetWriteMemBlk( int );
	#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
};


class MEM60 : public MEM6 {
private:
	// 
	static const MEMINFO IMAINROM;
	static const MEMINFO ICGROM1;
	static const MEMINFO IMAINRAM;
	
	BOOL useextram;						// ĥRAMȤ?
	
	void SetMB_60();					// ֥å(60Τ)
	
public:
	MEM60( BOOL );						// 󥹥ȥ饯
	~MEM60();							// ǥȥ饯
	
	BOOL Init( char *, BOOL );			// 
	void Reset();						// ꥻå
	
	// ľܥؿ
	BYTE ReadMainRam( WORD addr ){ return (addr<0x4000)&&(!useextram) ? 0xff : MainRam[addr&0x7fff]; }
	
	// ------------------------------------------
	BOOL DokoSave( cIni * );	// ɤǤSAVE
	BOOL DokoLoad( cIni * );	// ɤǤLOAD
	// ------------------------------------------
};


class MEM62 : public MEM6, public Device {
protected:
	// 
	static const MEMINFO IMAINROM;
	static const MEMINFO ICGROM1;
	static const MEMINFO ICGROM2;
	static const MEMINFO IKANJI;
	static const MEMINFO IVOICE;
	static const MEMINFO IMAINRAM;
	
	BYTE *CGRom2;		// CG ROM2		60: -KB 62,66: 8KB
	BYTE *KanjiRom;		//  ROM		60: -KB 62,66:32KB
	BYTE *VoiceRom;		//  ROM	60: -KB 62,66:16KB
	
	BOOL cgrom;			// CG ROM  TRUE:N60⡼ FALSE:N60m⡼
	BOOL kj_rom;		// ROM,ROM  TRUE:ROM FALSE:ROM
	BOOL kj_LR;			// ROM  TRUE: FALSE:
	
	BYTE Rf0;			// ꥳȥ쥸
	BYTE Rf1;			// ꥳȥ쥸
	BYTE Rf2;			// ꥳȥ쥸
	
	void SetMB_R( BYTE, BYTE );			// ꡼ɻΥ֥å(62,66Τ)
	void SetMB_W( BYTE );				// 饤ȻΥ֥å(62,66Τ)
	void SetWait( BYTE );				// ꥢ
	BYTE GetWait();						// ꥢȼ
	
	void SelectCGrom( int );			// CG ROM (62,66Τ)
	void SelectKanjiRom( BYTE );		// ROM  ROM ڤؤ(62,66Τ)
	
	// ǥХ
	static const Descriptor descriptor;
	static const InFuncPtr  indef[];
	static const OutFuncPtr outdef[];
	const Descriptor* GetDesc() const { return &descriptor; }
	
	// I/Oؿ
	inline void OutC1H( int, BYTE data ){ SelectCGrom( (data>>1)&1 ); }
	inline void OutC2H( int, BYTE data ){ SelectKanjiRom( data ); }
	inline void OutC3H( int, BYTE data ){}
	inline void OutF0H( int, BYTE data ){ SetMB_R( data, Rf1 ); }
	inline void OutF1H( int, BYTE data ){ SetMB_R( Rf0, data ); }
	inline void OutF2H( int, BYTE data ){ SetMB_W( data ); }
	inline void OutF3H( int, BYTE data ){ SetWait( data ); }
	
	inline BYTE InF0H( int ){ return Rf0; }
	inline BYTE InF1H( int ){ return Rf1; }
	inline BYTE InF2H( int ){ return Rf2; }
	inline BYTE InF3H( int ){ return GetWait() | 0x1f; }
	
public:
	MEM62( const ID& );					// 󥹥ȥ饯
	virtual ~MEM62();					// ǥȥ饯
	
	virtual BOOL Init( char *, BOOL );	// 
	void Reset();						// ꥻå
	
	// 8255ϴϢؿ
	void SetCGBank( BOOL );				// CG ROM BANK ڤؤ
	
	// ľܥؿ
	BYTE ReadMainRom( WORD addr ) { return MainRom[addr&0x7fff]; }
	BYTE ReadMainRam( WORD addr ) { return MainRam[addr&0xffff]; }
	BYTE ReadCGrom2( WORD addr )  { return CGRom2[addr&0x1fff]; }
	BYTE ReadKanjiRom( WORD addr ){ return KanjiRom[addr&0x7fff]; }
	BYTE ReadVoiceRom( WORD addr ){ return VoiceRom[addr&0x3fff]; }
	
	// ǥХID
	enum IDOut{ outC1H=0, outC2H, outC3H, outF0H, outF1H, outF2H, outF3H };
	enum IDIn {  inF0H=0,  inF1H,  inF2H,  inF3H };
	
	// ------------------------------------------
	BOOL DokoSave( cIni * );	// ɤǤSAVE
	BOOL DokoLoad( cIni * );	// ɤǤLOAD
	// ------------------------------------------
};


class MEM66 : public MEM62 {
private:
	// 
	static const MEMINFO IMAINROM;
	static const MEMINFO ICGROM1;
	static const MEMINFO ICGROM2;
	static const MEMINFO IKANJI;
	static const MEMINFO IVOICE;
	static const MEMINFO IMAINRAM;
	
	void SelectKanjiRom( BYTE );		// ROM  ROM ڤؤ(62,66Τ)
	
public:
	MEM66( const ID& );					// 󥹥ȥ饯
	~MEM66();							// ǥȥ饯
	
	BOOL Init( char *, BOOL );			// 
};


#endif	// MEMORY_H_INCLUDED
